package cn.com.qmth.examcloud.service.examwork.api;

import cn.com.qmth.examcloud.service.examwork.dao.ExamScoreRepo;
import cn.com.qmth.examcloud.service.examwork.entity.ExamScore;
import cn.com.qmth.examcloud.service.examwork.service.ExamScoreService;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageRequest;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

/**
 * 考试分数服务API
 * Created by songyue on 17/1/13.
 */
@RestController
@RequestMapping("${app.api.root}")
public class ExamScoreApi {

    @Autowired
    ExamScoreRepo examScoreRepo;

    @Autowired
    ExamScoreService examScoreService;

    @ApiOperation(value="查询所有考试分数",notes = "分页")
    @GetMapping("/exam_score/all/{curPage}/{pageSize}")
    public ResponseEntity getAllExamScore(@ModelAttribute ExamScore examCriteria, @PathVariable Integer curPage, @PathVariable Integer pageSize){
        return examScoreService.getAllExamScore(examCriteria,new PageRequest(curPage,pageSize));
    }

    @ApiOperation(value="查询所有考试分数",notes = "不分页")
    @GetMapping("/exam_score/all")
    public ResponseEntity getAllExamScore(@ModelAttribute ExamScore examCriteria){
        return examScoreService.getAllExamScore(examCriteria);
    }

    @ApiOperation(value="按ID查询考试分数",notes = "ID查询")
    @GetMapping("/exam_score/{id}")
    public ResponseEntity<ExamScore> getExamScoreById(@PathVariable Long id){
        return examScoreService.getExamScoreById(id);
    }

    @ApiOperation(value="新增考试分数",notes = "新增")
    @PostMapping("/exam_score")
    public ResponseEntity addExamScore(@RequestBody ExamScore examScore){
        return examScoreService.saveExamScore(examScore);
    }

    @ApiOperation(value="更新考试分数",notes = "更新")
    @PutMapping("/exam_score")
    public ResponseEntity updateExamScore(@RequestBody ExamScore examScore){
        return examScoreService.saveExamScore(examScore);
    }

    @ApiOperation(value="按ID删除考试分数",notes = "删除")
    @DeleteMapping("/exam_score/{id}")
    public ResponseEntity deleteExamScore(@PathVariable Long id){
        return examScoreService.deleteExamScore(id);
    }
}
