package cn.com.qmth.examcloud.service.examwork.dao;

import java.util.List;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.data.repository.query.QueryByExampleExecutor;

import cn.com.qmth.examcloud.service.examwork.entity.ExamStudent;
import org.springframework.transaction.annotation.Transactional;

public interface ExamStudentRepo extends JpaRepository<ExamStudent, Long>, QueryByExampleExecutor<ExamStudent>, JpaSpecificationExecutor<ExamStudent> {
    Page<ExamStudent> findByExamId(Long examId, Pageable pageable);

    List<ExamStudent> findByExamId(Long examId);

    @Query("select s from ExamStudent s where s.exam.id=?1 group by s.courseCode order by s.courseCode")
    List<ExamStudent> findDistinctCourseCode(Long examId);

    @Query("select s from ExamStudent s where s.exam.id=?1 and s.courseCode = ?2 group by s.courseCode order by s.courseCode")
    List<ExamStudent> findDistinctCourseCode(Long examId, String courseCode);

    @Query("select s from ExamStudent s where s.exam.id=?1 group by s.courseCode order by s.courseCode")
    Page<ExamStudent> findDistinctCourseCode(Long examId, Pageable pageable);

    @Query("select s from ExamStudent s where s.exam.id=?1 and s.courseCode = ?2 group by s.courseCode order by s.courseCode")
    Page<ExamStudent> findDistinctCourseCode(Long examId, String courseCode, Pageable pageable);

    List<ExamStudent> findByIdIn(List<Long> ids);

    @Query("select s from ExamStudent s where s.exam.id=?1 and s.courseCode =?2 group by s.paperType")
    List<ExamStudent> findDistinctPaperType(Long examId, String courseCode);

    @Query("select s from ExamStudent s where s.exam.id = ?1 and s.courseCode = ?2 and (s.studentCode = ?3 or s.identityNumber = ?3)")
    ExamStudent findFirstByExamId(Long examId, String courseCode, String stuNumber);

    @Query("select count(s) from ExamStudent s where s.exam.id = ?1 and s.rootOrgId=?2 and s.identityNumber=?3 and s.courseCode=?4 and s.id <> ?5")
    int checkExamStuById(Long examId, Long rootOrgId, String identityNumber, String courseCode, Long id);

    @Query("select count(s) from ExamStudent s where s.exam.id = ?1 and s.rootOrgId=?2 and s.identityNumber=?3 and s.courseCode=?4")
    int checkExamStu(Long examId, Long rootOrgId, String identityNumber, String courseCode);

    @Transactional
    @Modifying
    @Query("update ExamStudent s set s.name = ?2,s.specialtyName  = ?3,s.examSite = ?4  where s.identityNumber = ?1")
    void updateById(String identityNumber, String name, String specialtyName, String examSite);

    @Query("select s from ExamStudent s where s.rootOrgId=?1 and s.courseCode=?2")
    List<ExamStudent> findByCourseCode(Long rootOrgId, String courseCode);

    List<ExamStudent> findByOrgId(Long orgId);

    @Transactional
    @Modifying
    @Query("delete from ExamStudent s where s.exam.id = ?1")
    void deleteByExamId(Long examId);

    int countByExamIdAndFinished(Long examId, Boolean finished);

    @Query(nativeQuery = true,value = "select org_id, org_name, finished, count(id) from ecs_exam_student  where exam_id = :examId group by org_id, org_name, finished")
    List<Object[]> countByCampusAndFinished(@Param("examId") Long examId);


    @Transactional
    @Modifying
    @Query("update ExamStudent e set e.finished = ?2 where e.id = ?1")
    void updateForFinished(Long id,Boolean finished);
    
    @Query(value = "SELECT * FROM ecs_exam_student t WHERE t.exam_id = ?1 order by id desc limit ?2,?3 ",nativeQuery = true)
    public List<ExamStudent> findByLimit(Long examId,Integer startLimit,Integer endLimit);
}
