package cn.com.qmth.examcloud.service.examwork.service;

import cn.com.qmth.examcloud.service.examwork.dao.ExamScoreRepo;
import cn.com.qmth.examcloud.service.examwork.entity.ExamScore;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.ExampleMatcher;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;

import static org.springframework.data.domain.ExampleMatcher.GenericPropertyMatchers.startsWith;

/**
 * Created by songyue on 17/1/14.
 */
@Service
public class ExamScoreService {
    @Autowired
    ExamScoreRepo examScoreRepo;

    /**
     * 获取所有考试分数(分页)
     * @param examCriteria
     * @param pageable
     * @return
     */
    public ResponseEntity getAllExamScore(ExamScore examCriteria, Pageable pageable){
        ExampleMatcher exampleMatcher = ExampleMatcher.matching()
                .withMatcher("name",startsWith());
        Example<ExamScore> examExamScoreple = Example.of(examCriteria, exampleMatcher);
        return new ResponseEntity(examScoreRepo.findAll(examExamScoreple,pageable), HttpStatus.OK);
    }

    /**
     * 获取所有考试分数
     * @param examCriteria
     * @return
     */
    public ResponseEntity getAllExamScore(ExamScore examCriteria){
        ExampleMatcher exampleMatcher = ExampleMatcher.matching()
                .withMatcher("name",startsWith());
        Example<ExamScore> examExamScoreple = Example.of(examCriteria, exampleMatcher);
        return new ResponseEntity(examScoreRepo.findAll(examExamScoreple), HttpStatus.OK);
    }

    /**
     * 按ID获取考试分数
     * @param scoreId
     * @return
     */
    public ResponseEntity<ExamScore> getExamScoreById(Long scoreId){
        return new ResponseEntity<ExamScore>(examScoreRepo.findOne(scoreId),HttpStatus.OK);
    }

    /**
     * 保存考试分数
     * @param exam
     * @return
     */
    public ResponseEntity saveExamScore(ExamScore exam){
        return new ResponseEntity(examScoreRepo.save(exam),HttpStatus.OK);
    }

    /**
     * 删除考试分数
     * @param examId
     * @return
     */
    public ResponseEntity deleteExamScore(Long examId){
        examScoreRepo.delete(examId);
        return new ResponseEntity(HttpStatus.OK);
    }

    /**
     * 删除所有考试分数
     * @return
     */
    public ResponseEntity deleteAllExamScore(){
        examScoreRepo.deleteAll();
        return new ResponseEntity(HttpStatus.OK);
    }
}
